/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#ifndef PROJECTION_H
#define PROJECTION_H


#include <QObject>
#include <QtPlugin>
#include <QTimer>
#include <QtDBus>
#include <QGSettings/QGSettings>
#include <QLabel>
#include <QRegExpValidator>
#include <ukcc/widgets/hoverwidget.h>
#include <ukcc/widgets/imageutil.h>
#include <ukcc/widgets/switchbutton.h>
#include "changeprojectionname.h"
//#include <ukcc/widgets/hoverbtn.h>
#include "kswitchbutton.h"
using namespace kdk;

#include <ukcc/interface/interface.h>
#include "../component/DrownLabel/drownlabel.h"

namespace Ui {
class Projection;
}

class Projection : public QObject, CommonInterface
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "org.ukcc.CommonInterface")
    Q_INTERFACES(CommonInterface)

public:
    Projection();
    ~Projection();

    QString plugini18nName() Q_DECL_OVERRIDE;
    int pluginTypes() Q_DECL_OVERRIDE;
    QWidget * pluginUi() Q_DECL_OVERRIDE;
    const QString name() const  Q_DECL_OVERRIDE;
    bool isShowOnHomePage() const Q_DECL_OVERRIDE;
    QIcon icon() const Q_DECL_OVERRIDE;
    bool isEnable() const Q_DECL_OVERRIDE;
    QString translationPath() const Q_DECL_OVERRIDE;

    bool getWifiStatus();
    void setWifiStatus(bool status);
    void debusFileConfig();
    void deleteAllitemsOfLayout(QLayout* layout);

public:
    void initComponent();
    void showChangeProjectionNameDialog();
    void changeProjectionName(QString name);

protected:
    bool eventFilter(QObject *watched, QEvent *event);

private:
    Ui::Projection *ui;
    QString pluginName;
    int pluginType;
    QWidget * pluginWidget;
    HoverWidget * addWgt;
    KSwitchButton *projectionBtn;
    KSwitchButton *pincodeBtn;
    KSwitchButton *miracastBtn;
    DrownLabel *dropDownLabel = nullptr;
    QLabel *m_pin;
    bool enter = false;
    QString hostName;
    QMap<QString,QString>deviceName;


private:
    QTimer * pTimer;
    QDBusInterface *m_pServiceInterface;
    bool m_bbluetoothStatus;
    QGSettings * qtSettings;
    bool m_autoclose = false;
    void catchsignal();
    void delaymsec(int msec);
    int get_process_status(void);
    int get_process_source_status(void);
    int get_pin_button_status(void);
    void init_button_status(int);
    void init_pin_button_status(int);
    void init_source_button_status(int);
    void initSearchText();      // 搜索翻译
    int pinCodeStatus(int);
    void setLabelText(QLabel *label,QString text);
    void getSystemFontFamily();
    bool key = true;
    bool config_status = false;

public slots:
    void projectionButtonClickSlots(bool status);
    void pincodeButtonClickSlots(bool status);
    void miracastsourceButtonClickSlots(bool status);
    void pushButtonClickSlots();
    void onDrownLabelClicked();
    void projectionPinSlots(QString type,QString pin);
    void netPropertiesChangeSlot(QMap<QString, QVariant> property);

Q_SIGNALS:
    void resize();
};

#endif // PROJECTION_H
